#!/usr/bin/python2

import os
import sys
import subprocess
import semver


MIN_SEMVER = '0.32.2'
NEXT_SEMVER = 'next'
SCRIPTS_DIR = '/opt/edge/migrations'


class AppError(Exception):
    pass


def log(log_text):
    """
    :type log_text: str
    """
    print '[migrate]', log_text


def mk_migration_plan(last_version):
    """
    :type last_version: str
    :rtype: list[str]
    """
    migration_dirs = os.listdir(SCRIPTS_DIR)
    migration_plan = []
    for version in migration_dirs:
        if version == NEXT_SEMVER:
            continue

        if semver.compare(version, last_version) > 0:
            migration_plan.append(version)

    migration_plan.sort(semver.compare)
    if NEXT_SEMVER in migration_dirs:
        migration_plan.append(NEXT_SEMVER)

    return migration_plan


def main():
    if len(sys.argv) == 2:
        last_dch_ver = sys.argv[1]
        last_sem_ver = last_dch_ver.split('-')[0]

    elif len(sys.argv) == 1:
        log('DONE: clean installation')
        return

    else:
        log('FAIL: invalid command line')
        raise AppError

    if semver.compare(last_sem_ver, MIN_SEMVER) < 0:
        log('FAIL: manual migration required')
        return

    migration_plan = mk_migration_plan(last_sem_ver)
    log('EXEC: plan {}'.format(migration_plan))
    for version in migration_plan:
        log('EXEC: {}'.format(version))
        dirpath = os.path.join(SCRIPTS_DIR, version)

        runparts = subprocess.Popen(
            '. /opt/edge/rc.setenv; /bin/run-parts ' + dirpath,
            stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)

        stdout, stderr = runparts.communicate()
        if runparts.returncode:
            log('FAIL: {}'.format(stderr))
            raise AppError

    log('DONE')


if __name__ == '__main__':
    exit_code = 1
    try:
        main()
        exit_code = 0
    except AppError:
        pass

    sys.exit(exit_code)
